<?php
/**
 * Creates a special page that allows parses and returns the output of any wiki text
 * This is especially useful with wikitext that has dynamic output
 *
 * Author: Austin Che <http://openwetware.org/wiki/User:Austin>
 */

$wgExtensionFunctions[] = "wfExtensionSpecialParse";
$wgExtensionCredits['specialpage'][] = array(
                                             'name' => 'SpecialParse',
                                             'version' => '2007/04/24',
                                             'author' => 'Austin Che',
                                             'url' => 'http://openwetware.org/wiki/User:Austin/Extensions/SpecialParse',
                                             'description' => 'Special page for parsing wiki text',
                                             );
function wfExtensionSpecialParse() 
{
    global $wgMessageCache, $wgSpecialPages;
    $wgMessageCache->addMessages(array('parse' => 'Wikitext Parser', 'parsetext' => 'Parse', 
                                       'parsegeturl' => 'Get URL', 'parsegetraw' => 'Raw', 
                                       'parsepagetitle' => "Page Title"));
    $wgSpecialPages['Parse'] = array('SpecialPage', 'Parse');
}

function wfSpecialParse($par)
{
    global $wgOut, $wgRequest, $wgRawHtml, $wgUser;
    $oldRawHtml = $wgRawHtml;
    $wgRawHtml = false;         // disable raw html if it's enabled as this could be XSS security risk

    $mytitle = Title::makeTitle(NS_SPECIAL, "Parse");

    $wikitext = $wgRequest->getText('wikitext') ? $wgRequest->getText('wikitext') : $par;
    if ($wikitext)
    {
        $html = $wgOut->parse($wikitext, false);

        $mode = $wgRequest->getText('mode');
        if ($mode == "raw")
        {
            echo $html;
            $wgOut->disable();
        }
        else
        {
            $title = $wgRequest->getText('pagetitle');
            if ($title)
                $wgOut->setPageTitle($title);
            $wgOut->addHTML($html);
            $skin = $wgUser->getSkin();

            if ($wgRequest->getBool('parse'))
            {
                $params = "wikitext=" . urlencode($wikitext);
                if ($title)
                    $params .= "&pagetitle=" . urlencode($title);
                $wgOut->addHTML("<hr>" . $skin->makeLinkObj($mytitle, wfMsg('parsegeturl'), $params) . " | " .
                                $skin->makeLinkObj($mytitle, wfMsg('parsegetraw'), "mode=raw&" . $params) .
                                "<hr><p>");
            }
        }
    }
    else
    {
        $action = $mytitle->escapeLocalUrl();
        $wgOut->addHTML("<form id=\"wikitextparser\" method=\"post\" action=\"{$action}\">" . wfMsg("parsepagetitle") .
                        " <input type='text' name='pagetitle' /><p>" .
                        "<textarea rows='10' name=\"wikitext\">$wikitext</textarea>" .
                        "<input type='submit' name='parse' value='" . wfMsg('parsetext') . "' /></form>\n");
    }

    $wgRawHtml = $oldRawHtml;   // this probably does nothing useful
}
?>